@@ -9,13 +9,12 @@
 #include <caml/fail.h>
 #include <caml/bigarray.h>
 #include <caml/callback.h>
+#include <caml/unixsupport.h>
+#include <caml/socketaddr.h>
 
 //#define D(x) x
 #define D(x) while(0){}
 
-#define UNIX_BUFFER_SIZE 16384
-#define Nothing ((value) 0)
-
 typedef struct
 {
   OVERLAPPED overlapped;
@@ -23,44 +22,36 @@ typedef struct
   long action;
 } completionData;
 
-struct filedescr {
-  union {
-    HANDLE handle;
-    SOCKET socket;
-  } fd;
-  enum { KIND_HANDLE, KIND_SOCKET } kind;
-  int crt_fd;
-};
-#define Handle_val(v) (((struct filedescr *) Data_custom_val(v))->fd.handle)
-#define Socket_val(v) (((struct filedescr *) Data_custom_val(v))->fd.socket)
+#define Array_data(a, i) (((char *) Caml_ba_data_val(a)) + Long_val(i))
 
-extern void win32_maperr (DWORD errcode);
-extern void uerror (char * cmdname, value arg);
-extern value unix_error_of_code (int errcode);
-extern value win_alloc_handle (HANDLE h);
-extern value win_alloc_socket(SOCKET);
-extern void get_sockaddr (value mladdr,
-                          struct sockaddr * addr /*out*/,
-                          int * addr_len /*out*/);
+#ifndef Bytes_val
+#define Bytes_val(x) ((unsigned char *) Bp_val(x))
+#endif
 
-#define Array_data(a, i) (((char *) a->data) + Long_val(i))
+CAMLprim value ml_blit_bytes_to_buffer
+(value s, value i, value a, value j, value l)
+{
+  CAMLparam5(s, i, a, j, l);
+  unsigned char *src = Bytes_val(s) + Long_val(i);
+  char *dest = Array_data(a, j);
+  memcpy(dest, src, Long_val(l));
+  CAMLreturn(Val_unit);
+}
 
 CAMLprim value ml_blit_string_to_buffer
 (value s, value i, value a, value j, value l)
 {
-  char *src = String_val(s) + Int_val(i);
-  char *dest = Array_data(Bigarray_val(a), j);
-  memcpy(dest, src, Long_val(l));
-  return Val_unit;
+  return ml_blit_bytes_to_buffer(s, i, a, j, l);
 }
 
-CAMLprim value ml_blit_buffer_to_string
+CAMLprim value ml_blit_buffer_to_bytes
 (value a, value i, value s, value j, value l)
 {
-  char *src = Array_data(Bigarray_val(a), i);
-  char *dest = String_val(s) + Long_val(j);
+  CAMLparam5(a, i, s, j, l);
+  char *src = Array_data(a, i);
+  unsigned char *dest = Bytes_val(s) + Long_val(j);
   memcpy(dest, src, Long_val(l));
-  return Val_unit;
+  CAMLreturn(Val_unit);
 }
 
 /****/
@@ -88,7 +79,7 @@ static void invoke_completion_callback
     win32_maperr (errCode);
     err = unix_error_of_code(errno);
   }
-  name = copy_string (action_name[action]);
+  name = caml_copy_string(action_name[action]);
   D(printf("Action %s completed: id %ld -> len %ld / err %d (errCode %ld)\n",
            action_name[action], id, len, errno, errCode));
   args[0] = Val_long(id);
@@ -97,6 +88,7 @@ static void invoke_completion_callback
   args[3] = name;
   caml_callbackN(completionCallback, 4, args);
   D(printf("Callback performed\n"));
+  CAMLreturn0;
 }
 
 typedef struct {
@@ -243,7 +235,6 @@ static VOID CALLBACK perform_io_on_thread(ULONG_PTR param) {
 
 static void thread_io
 (long action, long id, value threads, HANDLE h, char * buf, long len) {
-  struct caml_bigarray *buf_arr = Bigarray_val(buf);
   ioInfo * info = GlobalAlloc(GPTR, sizeof(ioInfo));
   if (info == NULL) {
     errno = ENOMEM;
@@ -314,29 +305,27 @@ CAMLprim value win_wrap_overlapped (value fd) {
 
 CAMLprim value win_read
 (value fd, value buf, value ofs, value len, value id) {
-  CAMLparam4(fd, buf, ofs, len);
-  struct caml_bigarray *buf_arr = Bigarray_val(buf);
+  CAMLparam5(fd, buf, ofs, len, id);
 
   if (Field(fd, 1) == Val_long(0))
     overlapped_action (READ_OVERLAPPED, Long_val(id), Handle(fd),
-                       Array_data (buf_arr, ofs), Long_val(len));
+                       Array_data(buf, ofs), Long_val(len));
   else
     thread_io (READ, Long_val(id), Field(fd, 1), Handle(fd),
-               Array_data (buf_arr, ofs), Long_val(len));
+               Array_data(buf, ofs), Long_val(len));
   CAMLreturn (Val_unit);
 }
 
 CAMLprim value win_write
 (value fd, value buf, value ofs, value len, value id) {
-  CAMLparam4(fd, buf, ofs, len);
-  struct caml_bigarray *buf_arr = Bigarray_val(buf);
+  CAMLparam5(fd, buf, ofs, len, id);
 
   if (Field(fd, 1) == Val_long(0))
     overlapped_action (WRITE_OVERLAPPED, Long_val(id), Handle(fd),
-                       Array_data (buf_arr, ofs), Long_val(len));
+                       Array_data(buf, ofs), Long_val(len));
   else
     thread_io (WRITE, Long_val(id), Field(fd, 1), Handle(fd),
-               Array_data (buf_arr, ofs), Long_val(len));
+               Array_data(buf, ofs), Long_val(len));
   CAMLreturn (Val_unit);
 }
 
@@ -356,28 +345,29 @@ static void after_connect (SOCKET s) {
 static HANDLE events[MAXIMUM_WAIT_OBJECTS];
 //static OVERLAPPED oData[MAXIMUM_WAIT_OBJECTS];
 
-CAMLprim value win_register_wait (value socket, value kind, value idx) {
-  CAMLparam3(socket, kind, idx);
-  long i = Long_val(idx);
+CAMLprim value win_register_wait (value socket, value kind) {
+  CAMLparam2(socket, kind);
+  HANDLE h;
   long mask;
 
-  D(printf("Register: i %ld, kind %ld\n", Long_val(i), Long_val(kind)));
-  events[i] = CreateEvent(NULL, TRUE, FALSE, NULL);
+  D(printf("Register: %lx, kind %ld\n", (long)(Socket_val(socket)), Long_val(kind)));
+  h = CreateEvent(NULL, TRUE, FALSE, NULL);
   mask = (Long_val(kind) == 0) ? FD_CONNECT : FD_ACCEPT;
-  if (WSAEventSelect(Socket_val(socket), events[i], mask) == SOCKET_ERROR) {
+  if (WSAEventSelect(Socket_val(socket), h, mask) == SOCKET_ERROR) {
     win32_maperr(WSAGetLastError ());
     uerror("WSAEventSelect", Nothing);
   }
 
-  CAMLreturn (Val_unit);
+  CAMLreturn(win_alloc_handle(h));
 }
 
-CAMLprim value win_check_connection (value socket, value kind, value idx) {
-  CAMLparam3 (socket, kind, idx);
+CAMLprim value win_check_connection (value socket, value kind, value h) {
+  CAMLparam3 (socket, kind, h);
   WSANETWORKEVENTS evs;
-  int res, err, i = Long_val(idx);
+  int res, err;
 
-  D(printf("Check connection... %d\n", i));
+  D(printf("Check connection... socket = %lx; h = %lx\n",
+              (long)(Socket_val(socket)), Handle_val(h)));
   if (WSAEnumNetworkEvents(Socket_val(socket), NULL, &evs)) {
     win32_maperr(WSAGetLastError ());
     uerror("WSAEnumNetworkEvents", Nothing);
@@ -386,7 +376,7 @@ CAMLprim value win_check_connection (value socket, value kind, value idx) {
     win32_maperr(WSAGetLastError ());
     uerror("WSAEventSelect", Nothing);
   }
-  if (!CloseHandle(events[i])) {
+  if (!CloseHandle(Handle_val(h))) {
     win32_maperr(GetLastError ());
     uerror("CloseHandle", Nothing);
   }
@@ -410,7 +400,7 @@ CAMLprim value init_lwt (value callback) {
   int i;
 
   D(printf("Init...\n"));
-  register_global_root (&completionCallback);
+  caml_register_global_root(&completionCallback);
   completionCallback = callback;
 
   dummyEvent = CreateEvent(NULL, TRUE, FALSE, NULL);  // Dummy event
@@ -433,14 +423,16 @@ CAMLprim value init_lwt (value callback) {
   CAMLreturn (Val_long (MAXIMUM_WAIT_OBJECTS));
 }
 
-CAMLprim value win_wait (value timeout, value event_count) {
-  CAMLparam2(timeout, event_count);
+CAMLprim value win_wait (value timeout, value event_list) {
+  CAMLparam2(timeout, event_list);
   DWORD t, t2;
   DWORD res;
-  long ret, n = Long_val(event_count);
+  long ret, n;
   t = Long_val(timeout);
   if (t < 0) t = INFINITE;
   t2 = (compN > 0) ? 0 : t;
+  for (n = 0; event_list != Val_emptylist; event_list = Field(event_list, 1))
+    events[n++] = Handle_val(Field(event_list, 0));
   D(printf("Waiting: %ld events, timeout %ldms -> %ldms\n", n, t, t2));
   res =
     (n > 0) ?
@@ -513,7 +505,7 @@ value win_pipe(long readMode, long writeMode) {
 
   readfd = win_alloc_handle(readh);
   writefd = win_alloc_handle(writeh);
-  res = alloc_small(2, 0);
+  res = caml_alloc_small(2, 0);
   Store_field(res, 0, readfd);
   Store_field(res, 1, writefd);
   CAMLreturn (res);
@@ -530,7 +522,7 @@ CAMLprim value win_pipe_out (value unit) {
 }
 
 static int socket_domain_table[] = {
-  PF_UNIX, PF_INET
+  PF_UNIX, PF_INET, PF_INET6
 };
 
 static int socket_type_table[] = {
@@ -610,15 +602,14 @@ static int notify_filter_flags[8] = {
 CAMLprim value win_readdirtorychanges
 (value fd_val, value buf_val, value recursive, value flags, value id_val) {
   CAMLparam5(fd_val, buf_val, recursive, flags, id_val);
-  struct caml_bigarray *buf_arr = Bigarray_val(buf_val);
   long id = Long_val(id_val);
   HANDLE fd = Handle_val(fd_val);
-  char * buf = Array_data (buf_arr, 0);
-  long len = buf_arr->dim[0];
+  char * buf = Array_data(buf_val, 0);
+  long len = Caml_ba_array_val(buf_val)->dim[0];
   long action = READDIRECTORYCHANGES;
   BOOL res;
   long err;
-  int notify_filter = convert_flag_list(flags, notify_filter_flags);
+  int notify_filter = caml_convert_flag_list(flags, notify_filter_flags);
   completionData * d = GlobalAlloc(GPTR, sizeof(completionData));
   if (d == NULL) {
     errno = ENOMEM;
@@ -648,8 +639,7 @@ CAMLprim value win_readdirtorychanges
 CAMLprim value win_parse_directory_changes (value buf_val) {
   CAMLparam1(buf_val);
   CAMLlocal4(lst, tmp, elt, filename);
-  struct caml_bigarray *buf_arr = Bigarray_val(buf_val);
-  char * pos = Array_data (buf_arr, 0);
+  char * pos = Array_data(buf_val, 0);
   FILE_NOTIFY_INFORMATION * entry;
 
   lst = Val_long(0);
@@ -657,7 +647,7 @@ CAMLprim value win_parse_directory_changes (value buf_val) {
     entry = (FILE_NOTIFY_INFORMATION *)pos;
     elt = caml_alloc_tuple(2);
     filename = caml_alloc_string(entry->FileNameLength);
-    memmove(String_val(filename), entry->FileName, entry->FileNameLength);
+    memmove((char *)String_val(filename), entry->FileName, entry->FileNameLength);
     Store_field (elt, 0, filename);
     Store_field (elt, 1, Val_long(entry->Action - 1));
     tmp = caml_alloc_tuple(2);
@@ -686,3 +676,15 @@ CAMLprim value win_open_directory (value path, value wpath) {
   }
   CAMLreturn(win_alloc_handle(h));
 }
+
+value copy_wstring(LPCWSTR s);
+
+CAMLprim value win_long_path_name(value path) {
+  CAMLparam1(path);
+  wchar_t lbuf[32768] = L"";
+  DWORD res;
+
+  res = GetLongPathNameW((LPCWSTR) String_val(path), lbuf, 32768);
+
+  CAMLreturn(res == 0 || res > 32767 ? path : copy_wstring(lbuf));
+}
