@@ -30,7 +30,8 @@ let localCanonicalHostName =
 
 let clientHostName : string Prefs.t =
   Prefs.createString "clientHostName" localCanonicalHostName
-    "!set host name of client"
+    ~category:(`Advanced `Remote)
+    "set host name of client"
     ("When specified, the host name of the client will not be guessed " ^
      "and the provided host name will be used to find the archive.")
 
