@@ -32,7 +32,8 @@ let isMacOSX = isMacOSXPred ()
 
 let rsrcSync =
   Prefs.createBoolWithDefault "rsrc"
-    "!synchronize resource forks (true/false/default)"
+    ~category:(`Advanced `Sync)
+    "synchronize resource forks (true/false/default)"
     "When set to {\\tt true}, this flag causes Unison to synchronize \
      resource forks and HFS meta-data.  On filesystems that do not \
      natively support resource forks, this data is stored in \
@@ -46,6 +47,7 @@ let rsrcSync =
    to the other host during initialization *)
 let rsrc =
   Prefs.createBool "rsrc-aux" false
+    ~category:(`Internal `Pseudo)
     "*synchronize resource forks and HFS meta-data" ""
 
 let init b =
@@ -179,8 +181,21 @@ type 'a ressInfo =
   | HfsRess of Uutil.Filesize.t
   | AppleDoubleRess of int * float * float * Uutil.Filesize.t * 'a
 
+let mressInfo m = Umarshal.(sum3 unit Uutil.Filesize.m
+                              (prod5 int float float Uutil.Filesize.m m id id)
+                              (function
+                               | NoRess -> I31 ()
+                               | HfsRess a -> I32 a
+                               | AppleDoubleRess (a, b, c, d, e) -> I33 (a, b, c, d, e))
+                              (function
+                               | I31 () -> NoRess
+                               | I32 a -> HfsRess a
+                               | I33 (a, b, c, d, e) -> AppleDoubleRess (a, b, c, d, e)))
+
 type ressStamp = unit ressInfo
 
+let mressStamp = mressInfo Umarshal.unit
+
 let ressStampToString r =
   match r with
     NoRess         ->
@@ -195,6 +210,10 @@ type info =
   { ressInfo : (Fspath.t * int64) ressInfo;
     finfo : string }
 
+let minfo = Umarshal.(prod2 (mressInfo (prod2 Fspath.m int64 id id)) string
+                        (fun {ressInfo; finfo} -> ressInfo, finfo)
+                        (fun (ressInfo, finfo) -> {ressInfo; finfo}))
+
 external getFileInfosInternal :
   System.fspath -> bool -> string * int64 = "getFileInfos"
 external setFileInfosInternal :
