@@ -98,11 +98,10 @@ let compile_pattern clause =
     end in
   (compiled, v)
 
-let create name ?(local=false) ?(advanced=false) fulldoc =
+let create name ~category ?(local=false) fulldoc =
   let pref =
-    Prefs.create name ~local []
-      ((if advanced then "!" else "")
-       ^ "add a pattern to the " ^ name ^ " list")
+    Prefs.create name ~category ~local []
+      ("add a pattern to the " ^ name ^ " list")
       fulldoc
       (fun oldList string ->
          ignore (compile_pattern string); (* Check well-formedness *)
