@@ -496,7 +496,18 @@ let safeMarshal marshalPayload tag data rem =
             let start = String.escaped start in
             Util.msg "send [%s] '%s' %d bytes\n"
               (Bytearray.to_string tag) start length);
-  (encodeInt (l + length) :: (tag, 0, l) :: rem')
+  let len = l + length in
+  if (len lsr 31) lsr 1 <> 0 then (* [encodeInt] can only encode 32 bits *)
+    raise (Util.Fatal
+            "Protocol error: message data too big. This may be a bug or it\n\
+             may be that your replicas are huge and the amount of updates\n\
+             can't be handled by the current protocol implementation. If you\n\
+             believe it is a bug then please consider reporting it.\n\
+             Otherwise, try reducing the amount of updates by syncing the\n\
+             replicas in smaller steps (using the \"path\" preference, for\n\
+             example). You may have to do this for the initial sync only.")
+  else
+    (encodeInt len :: (tag, 0, l) :: rem')
 
 let safeUnmarshal unmarshalPayload tag buf =
   let taglength = Bytearray.length tag in
@@ -558,29 +569,15 @@ let makeMarshalingFunctions payloadMarshalingFunctions string =
    these be part of it too? *)
 let sshCmd =
   Prefs.createString "sshcmd" "ssh"
-    ("!path to the ssh executable")
+    ~category:(`Advanced `Remote)
+    ("path to the ssh executable")
     ("This preference can be used to explicitly set the name of the "
      ^ "ssh executable (e.g., giving a full path name), if necessary.")
 
-let rshCmd =
-  Prefs.createString "rshcmd" "rsh"
-    ("*path to the rsh executable")
-    ("This preference can be used to explicitly set the name of the "
-     ^ "rsh executable (e.g., giving a full path name), if necessary.")
-
-let rshargs =
-  Prefs.createString "rshargs" ""
-    "*other arguments (if any) for remote shell command"
-    ("The string value of this preference will be passed as additional "
-     ^ "arguments (besides the host name and the name of the Unison "
-     ^ "executable on the remote system) to the \\verb|rsh| "
-     ^ "command used to invoke the remote server. The backslash is an "
-     ^ "escape character."
-     )
-
 let sshargs =
   Prefs.createString "sshargs" ""
-    "!other arguments (if any) for remote shell command"
+    ~category:(`Advanced `Remote)
+    "other arguments (if any) for remote shell command"
     ("The string value of this preference will be passed as additional "
      ^ "arguments (besides the host name and the name of the Unison "
      ^ "executable on the remote system) to the \\verb|ssh| "
@@ -588,16 +585,22 @@ let sshargs =
      ^ "escape character."
      )
 
+(* rsh prefs removed since 2.52 *)
+let () = Prefs.markRemoved "rshcmd"
+let () = Prefs.markRemoved "rshargs"
+
 let serverCmd =
   Prefs.createString "servercmd" ""
-    ("!name of " ^ Uutil.myName ^ " executable on remote server")
+    ~category:(`Advanced `Remote)
+    ("name of " ^ Uutil.myName ^ " executable on remote server")
     ("This preference can be used to explicitly set the name of the "
      ^ "Unison executable on the remote server (e.g., giving a full "
      ^ "path name), if necessary.")
 
 let addversionno =
   Prefs.createBool "addversionno" false
-    ("!add version number to name of " ^ Uutil.myName ^ " on server")
+    ~category:(`Advanced `Remote)
+    ("add version number to name of " ^ Uutil.myName ^ " on server")
     ("When this flag is set to {\\tt true}, Unison "
      ^ "will use \\texttt{unison-\\ARG{currentmajorversionnumber}} instead of "
      ^ "just \\verb|unison| as the remote server command (note that the minor "
@@ -950,7 +953,9 @@ let streamReg = Lwt_util.make_region 1
 
 let streamingActivated =
   Prefs.createBool "stream" true
-    ("!use a streaming protocol for transferring file contents")
+    ~category:(`Advanced `Remote)
+    ~deprecated:true
+    ("use a streaming protocol for transferring file contents")
     "When this preference is set, Unison will use an experimental \
      streaming protocol for transferring file contents more efficiently. \
      The default value is \\texttt{true}."
@@ -1355,14 +1360,13 @@ let checkServerUpgrade conn header =
 *)
 let halfduplex =
   Prefs.createBool "halfduplex" false
-    "!force half-duplex communication with the server"
+    ~category:(`Advanced `Remote)
+    ~deprecated:true
+    "force half-duplex communication with the server"
     "When this flag is set to {\\tt true}, Unison network communication \
      is forced to be half duplex (the client and the server never \
      simultaneously emit data).  If you experience unstabilities with \
-     your network link, this may help.  The communication is always \
-     half-duplex when synchronizing with a Windows machine due to a \
-     limitation of Unison current implementation that could result \
-     in a deadlock."
+     your network link, this may help."
 
 let negociateFlowControlLocal conn () =
   disableFlowControl conn.outputQueue;
@@ -1444,7 +1448,8 @@ let buildSocket host port kind ?(err="") ai =
                   if ai.Unix.ai_family = Unix.PF_INET6 then
                     Lwt_unix.setsockopt socket Unix.IPV6_ONLY true;
                   (* Allow reuse of local addresses for bind *)
-                  Lwt_unix.setsockopt socket Unix.SO_REUSEADDR true;
+                  if ai.Unix.ai_family <> Unix.PF_UNIX then
+                    Lwt_unix.setsockopt socket Unix.SO_REUSEADDR true;
                   (* Bind the socket to portnum on the local host
                      or to a filesystem path (when Unix domain socket) *)
                   Lwt_unix.bind socket ai.Unix.ai_addr;
@@ -1588,15 +1593,11 @@ let buildShellConnection onClose shell host userOpt portOpt rootName termInterac
   let shellCmd =
     (if shell = "ssh" then
       Prefs.read sshCmd
-    else if shell = "rsh" then
-      Prefs.read rshCmd
     else
       shell) in
   let shellCmdArgs =
     (if shell = "ssh" then
       Prefs.read sshargs
-    else if shell = "rsh" then
-      Prefs.read rshargs
     else
       "") in
   let preargs =
@@ -1870,15 +1871,11 @@ let openConnectionStart clroot =
           let shellCmd =
             (if shell = "ssh" then
               Prefs.read sshCmd
-            else if shell = "rsh" then
-              Prefs.read rshCmd
             else
               shell) in
           let shellCmdArgs =
             (if shell = "ssh" then
               Prefs.read sshargs
-            else if shell = "rsh" then
-              Prefs.read rshargs
             else
               "") in
           let preargs =
@@ -2090,7 +2087,8 @@ let commandLoop ~compatMode in_ch out_ch =
 
 let killServer =
   Prefs.createBool "killserver" false
-    "!kill server when done (even when using sockets)"
+    ~category:(`Advanced `Remote)
+    "kill server when done (even when using sockets)"
     ("When set to \\verb|true|, this flag causes Unison to kill the remote "
      ^ "server process when the synchronization is finished.  This behavior "
      ^ "is the default for \\verb|ssh| connections, so this preference is not "
@@ -2117,20 +2115,23 @@ let waitOnPort hosts port =
        let hosts = match hosts with [] -> [""] | _ -> hosts in
        let listening = Lwt_unix.run (buildListenSocket hosts port) in
        let accepting = Array.make (Safelist.length listening) None in
-       let accept i l =
+       let rec accept i l =
          match accepting.(i) with
            | None ->
                let st = Lwt_unix.accept l >>= fun s -> Lwt.return (i, s) in
                let () = accepting.(i) <- Some st in
                st
            | Some st -> st
-       and serve (i, s) = accepting.(i) <- None; s in
+       and serve (i, s) = accepting.(i) <- None; setKeepalive s; s
+       and setKeepalive = function
+         | (_, Unix.ADDR_UNIX _) -> ()
+         | (c, ADDR_INET _) -> Lwt_unix.setsockopt c Unix.SO_KEEPALIVE true
+       in
        Util.msg "server started\n";
        let rec handleClients () =
          let (connected, _) =
            serve @@ Lwt_unix.run (Lwt.choose (List.mapi accept listening))
          in
-         Lwt_unix.setsockopt connected Unix.SO_KEEPALIVE true;
          begin try
            (* Accept a connection *)
            let compatMode = Some (if is248Exe then "2.48" else "2.51") in
