@@ -25,7 +25,8 @@ let verbose = Util.debug "stasher+"
 
 let backuplocation =
   Prefs.createString "backuploc" "central"
-    "!where backups are stored ('local' or 'central')"
+    ~category:(`Advanced `Syncprocess)
+    "where backups are stored ('local' or 'central')"
     ("This preference determines whether backups should be kept locally, near the "
      ^ "original files, or"
      ^" in a central directory specified by the \\texttt{backupdir} "
@@ -36,7 +37,8 @@ let backuplocation =
 let _ = Prefs.alias backuplocation "backuplocation"
 
 let backup =
-  Pred.create "backup" ~advanced:true
+  Pred.create "backup"
+    ~category:(`Advanced `Syncprocess)
     ("Including the preference \\texttt{-backup \\ARG{pathspec}} "
      ^ "causes Unison to keep backup files for each path that matches "
      ^ "\\ARG{pathspec}; directories (nor their permissions or any other "
@@ -51,7 +53,8 @@ let backup =
 let _ = Pred.alias backup "mirror"
 
 let backupnot =
-  Pred.create "backupnot" ~advanced:true
+  Pred.create "backupnot"
+    ~category:(`Advanced `Syncprocess)
     ("The values of this preference specify paths or individual files or"
      ^ " regular expressions that should {\\em not} "
      ^ "be backed up, even if the {\\tt backup} preference selects "
@@ -65,13 +68,14 @@ let shouldBackup p =
 
 let backupprefix =
   Prefs.createString "backupprefix" ".bak.$VERSION."
-    "!prefix for the names of backup files"
+    ~category:(`Advanced `Syncprocess)
+    "prefix for the names of backup files"
     ("When a backup for a file \\verb|NAME| is created, it is stored "
      ^ "in a directory specified by \\texttt{backuplocation}, in a file called "
      ^ "\\texttt{backupprefix}\\verb|NAME|\\texttt{backupsuffix}."
      ^ " \\texttt{backupprefix} can include a directory name (causing Unison to "
      ^ "keep all backup files for a given directory in a subdirectory with this name), and both "
-     ^ " \\texttt{backupprefix} and \\texttt{backupsuffix} can contain the string"
+     ^ " \\texttt{backupprefix} and \\texttt{backupsuffix} can contain the string "
      ^ "\\ARG{\\$VERSION}, which will be replaced by the \\emph{age} of the backup "
      ^ "(1 for the most recent, 2 for the second most recent, and so on...)."
      ^ " This keyword is ignored if it appears in a directory name"
@@ -87,12 +91,15 @@ let backupprefix =
 
 let backupsuffix =
   Prefs.createString "backupsuffix" ""
-    "!a suffix to be added to names of backup files"
+    ~category:(`Advanced `Syncprocess)
+    "a suffix to be added to names of backup files"
     ("See \\texttt{backupprefix} for full documentation.")
 
 let backups =
   Prefs.createBool "backups" false
-    "!keep backup copies of all files (see also 'backup')"
+    ~category:(`Advanced `Syncprocess)
+    ~deprecated:true
+    "keep backup copies of all files (see also 'backup')"
     ("Setting this flag to true is equivalent to "
      ^" setting \\texttt{backuplocation} to \\texttt{local}"
      ^" and \\texttt{backup} to \\verb|Name *|.")
@@ -113,7 +120,8 @@ let translateOldPrefs () =
 
 let maxbackups =
   Prefs.createInt "maxbackups" 2
-    "!number of backed up versions of a file"
+    ~category:(`Advanced `Syncprocess)
+    "number of backed up versions of a file"
     ("This preference specifies the number of backup versions that will "
      ^ "be kept by unison, for each path that matches the predicate "
      ^ "\\verb|backup|.  The default is 2.")
@@ -123,7 +131,8 @@ let _ = Prefs.alias maxbackups "backupversions"
 
 let backupdir =
   Prefs.createString "backupdir" ""
-    "!directory for storing centralized backups"
+    ~category:(`Advanced `Syncprocess)
+    "directory for storing centralized backups"
     ("If this preference is set, Unison will use it as the name of the "
      ^ "directory used to store backup files specified by "
      ^ "the {\\tt backup} preference, when {\\tt backuplocation} is set"
@@ -142,7 +151,8 @@ let backupDirectory () =
                (Some (System.fspathToString (Util.fileInUnisonDir "backup"))))
 
 let backupcurrent =
-  Pred.create "backupcurr" ~advanced:true
+  Pred.create "backupcurr"
+    ~category:(`Advanced `Syncprocess)
     ("Including the preference \\texttt{-backupcurr \\ARG{pathspec}} "
      ^" causes Unison to keep a backup of the {\\em current} version of every file "
      ^ "matching \\ARG{pathspec}.  "
@@ -154,7 +164,8 @@ let backupcurrent =
      ^ "\\sectionref{pathspec}{Path Specification}.")
 
 let backupcurrentnot =
-  Pred.create "backupcurrnot" ~advanced:true
+  Pred.create "backupcurrnot"
+    ~category:(`Advanced `Syncprocess)
    "Exceptions to \\verb|backupcurr|, like the \\verb|ignorenot| preference."
 
 let shouldBackupCurrent p =
@@ -265,19 +276,33 @@ let updateBackupNamingFunctions () =
 
 (*------------------------------------------------------------------------------------*)
 
-let makeBackupName path i =
+let makeBackupName fspath path i =
+  (* In the special case when the root itself is a file, use the root's name
+     as the backup file name. Empty path will break backups.
+     We only check the path being empty, and not its type, because the root
+     can change from file to dir and vice versa between syncs. *)
+  let path' =
+    if Path.isEmpty path then
+      Path.fromString (Filename.basename (Fspath.toString fspath))
+    else path in
+
   (* if backups are kept centrally, the current version has exactly
      the same name as the original, for convenience. *)
   if i=0 && Prefs.read backuplocation = "central" then
-    path
+    path'
   else
     Path.addSuffixToFinalName
-      (Path.addPrefixToFinalName path (!make_prefix i))
+      (Path.addPrefixToFinalName path' (!make_prefix i))
       (!make_suffix i)
 
-let stashDirectory fspath =
+let stashDirectory fspath path =
   match Prefs.read backuplocation with
     "central" -> backupDirectory ()
+  | "local" when Path.isEmpty path ->
+      (* Special case when the root itself is a file. Can't use the root
+         as the backup location, which must be a directory. Use the root's
+         parent instead. *)
+      Fspath.canonize (Some (Filename.dirname (Fspath.toString fspath)))
   | "local" -> fspath
   |  _ -> raise (Util.Fatal ("backuplocation preference should be set"
                              ^"to central or local."))
@@ -298,15 +323,15 @@ let showContent typ fspath path =
    sufficiently large number!
 *)
 let backupPath fspath path =
-  let sFspath = stashDirectory fspath in
+  let sFspath = stashDirectory fspath path in
 
-  let rec f path i =
-    let tempPath = makeBackupName path i in
+  let rec f fspath path i =
+    let tempPath = makeBackupName fspath path i in
     verbose (fun () -> Util.msg "backupPath f %s %d\n" (Path.toString path) i);
     if Os.exists sFspath tempPath then
       if i < Prefs.read maxbackups then begin
         verbose (fun () -> Util.msg "need to rename backup file\n");
-        Os.rename "backupPath" sFspath tempPath sFspath (f path (i + 1))
+        Os.rename "backupPath" sFspath tempPath sFspath (f fspath path (i + 1))
       end
       else if i >= Prefs.read maxbackups then
         Os.delete sFspath tempPath;
@@ -333,10 +358,10 @@ let backupPath fspath path =
                 will create a separate directory tree. *)
         if (Prefs.read backuplocation = "central") &&
           (Fileinfo.get false sFspath backdir).Fileinfo.typ != `DIRECTORY then
-          let backdir = f backdir 0 in
+          let backdir = f sFspath backdir 0 in
           Os.createDir sFspath backdir props in
 
-  let path0 = makeBackupName path 0 in
+  let path0 = makeBackupName fspath path 0 in
   let sourceTyp = (Fileinfo.get true fspath path).Fileinfo.typ in
   let path0Typ = (Fileinfo.get false sFspath path0).Fileinfo.typ in
 
@@ -358,7 +383,7 @@ let backupPath fspath path =
       (showContent path0Typ sFspath path0)
       (Fspath.toDebugString fspath) (Path.toString path)
       (showContent sourceTyp fspath path));
-    let sPath = f path 0 in
+    let sPath = f fspath path 0 in
     (* Make sure the parent directory exists *)
     begin match Path.deconstructRev sPath with
      | None -> mkdirectories Path.empty
@@ -464,9 +489,9 @@ let getRecentVersion fspath path fingerprint =
       (Path.toString path)
       (Fspath.toDebugString fspath));
   Util.convertUnixErrorsToTransient "getRecentVersion" (fun () ->
-    let dir = stashDirectory fspath in
+    let dir = stashDirectory fspath path in
     let rec aux_find i =
-      let path = makeBackupName path i in
+      let path = makeBackupName fspath path i in
       if Os.exists dir path &&
         (* FIX: should check that the existing file has the same size, to
            avoid computing the fingerprint if it is obviously going to be
