@@ -47,7 +47,9 @@ end
  **********************************************************************)
 
 let auto =
-  Prefs.createBool "auto" false "automatically accept default (nonconflicting) actions"
+  Prefs.createBool "auto" false
+    ~category:(`Basic `Syncprocess_CLI)
+    "automatically accept default (nonconflicting) actions"
     ("When set to {\\tt true}, this flag causes the user "
      ^ "interface to skip asking for confirmations on "
      ^ "non-conflicting changes.  (More precisely, when the user interface "
@@ -59,17 +61,20 @@ let auto =
    gets sent to the server at startup *)
 let mainWindowHeight =
   Prefs.createInt "height" 15
-    "!height (in lines) of main window in graphical interface"
+    ~category:(`Advanced `GUI)
+    "height (in lines) of main window in graphical interface"
     ("Used to set the height (in lines) of the main window in the graphical "
      ^ "user interface.")
 
 let expert =
   Prefs.createBool "expert" false
+    ~category:(`Internal `Devel)
     "*Enable some developers-only functionality in the UI" ""
 
 let profileLabel =
   Prefs.createString "label" ""
-    "!provide a descriptive string label for this profile"
+    ~category:(`Advanced `General)
+    "provide a descriptive string label for this profile"
     ("Used in a profile to provide a descriptive string documenting its "
      ^ "settings.  (This is useful for users that switch between several "
      ^ "profiles, especially using the `fast switch' feature of the "
@@ -77,7 +82,8 @@ let profileLabel =
 
 let profileKey =
   Prefs.createString "key" ""
-    "!define a keyboard shortcut for this profile (in some UIs)"
+    ~category:(`Advanced `General)
+    "define a keyboard shortcut for this profile (in some UIs)"
     ("Used in a profile to define a numeric key (0-9) that can be used in "
      ^ "the user interface to switch immediately to this profile.")
 (* This preference is not actually referred to in the code anywhere, since
@@ -87,7 +93,8 @@ let profileKey =
 
 let contactquietly =
   Prefs.createBool "contactquietly" false
-    "!suppress the 'contacting server' message during startup"
+    ~category:(`Advanced `General)
+    "suppress the 'contacting server' message during startup"
     ("If this flag is set, Unison will skip displaying the "
      ^ "`Contacting server' message (which some users find annoying) "
      ^ "during startup.")
@@ -97,7 +104,8 @@ let contactingServerMsg () =
 
 let repeat =
   Prefs.createString "repeat" ""
-    "!synchronize repeatedly (text interface only)"
+    ~category:(`Advanced `Syncprocess_CLI)
+    "synchronize repeatedly (text interface only)"
     ("Setting this preference causes the text-mode interface to synchronize "
      ^ "repeatedly, rather than doing it just once and stopping.  If the "
      ^ "argument is a number, Unison will pause for that many seconds before "
@@ -108,7 +116,8 @@ let repeatWatcher () = Prefs.read repeat = "watch"
 
 let retry =
   Prefs.createInt "retry" 0
-    "!re-try failed synchronizations N times (text ui only)"
+    ~category:(`Advanced `Syncprocess_CLI)
+    "re-try failed synchronizations N times (text ui only)"
     ("Setting this preference causes the text-mode interface to try again "
      ^ "to synchronize "
      ^ "updated paths where synchronization fails.  Each such path will be "
@@ -117,7 +126,8 @@ let retry =
 
 let confirmmerge =
   Prefs.createBool "confirmmerge" false
-    "!ask for confirmation before committing results of a merge"
+    ~category:(`Advanced `Syncprocess)
+    "ask for confirmation before committing results of a merge"
     ("Setting this preference causes both the text and graphical interfaces"
      ^ " to ask the user if the results of a merge command may be committed "
      ^ " to the replica or not. Since the merge command works on temporary files,"
@@ -128,7 +138,9 @@ let confirmmerge =
 let runTestsPrefName = "selftest"
 let runtests =
   Prefs.createBool runTestsPrefName false
-    "!run internal tests and exit"
+    ~category:`Expert
+    ~cli_only:true
+    "run internal tests and exit"
    ("Run internal tests and exit.  This option is mostly for developers and must be used "
   ^ "carefully: in particular, "
   ^ "it will delete the contents of both roots, so that it can install its own files "
@@ -155,6 +167,7 @@ let choose s1 s2 = if !Update.foundArchives then s1 else s2
 
 let showprev =
   Prefs.createBool "showprev" false
+    ~category:(`Internal `Devel)
     "*Show previous properties, if they differ from current"
     ""
 
@@ -538,7 +551,7 @@ let scanProfiles () =
           match prefs with
           | None -> None
           | Some prefs ->
-          let fileContents = Safelist.map (fun (_, _, n, v) -> (n, v)) prefs in
+          let fileContents = Safelist.map (fun (_, n, v) -> (n, v)) prefs in
           let roots =
             Safelist.map snd
               (Safelist.filter (fun (n, _) -> n = "root") fileContents) in
@@ -757,10 +770,13 @@ let refreshConnection ~displayWaitMessage ~termInteract =
 
 let anonymousArgs =
   Prefs.createStringList "rest"
+    ~category:(`Internal `Other)
     "*roots or profile name" ""
 
 let testServer =
   Prefs.createBool "testserver" false
+    ~category:(`Advanced `Remote)
+    ~cli_only:true
     "exit immediately after the connection to the server"
     ("Setting this flag on the command line causes Unison to attempt to "
      ^ "connect to the remote server and, if successful, print a message "
